# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ProtobufTrafficFlow_v5.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ProtobufTrafficFlow_v5.proto',
  package='trafficFlow',
  syntax='proto2',
  serialized_pb=_b('\n\x1cProtobufTrafficFlow_v5.proto\x12\x0btrafficFlow\"\xa2\x02\n\x10TrafficFlowGroup\x12\x35\n\x0fmetaInformation\x18\x01 \x01(\x0b\x32\x1c.trafficFlow.MetaInformation\x12-\n\x0btrafficFlow\x18\x02 \x03(\x0b\x32\x18.trafficFlow.TrafficFlow\x12I\n\x19trafficFlowWithPrediction\x18\x03 \x03(\x0b\x32&.trafficFlow.TrafficFlowWithPrediction\x12]\n#trafficFlowWithPredictionPerSection\x18\x04 \x03(\x0b\x32\x30.trafficFlow.TrafficFlowWithPredictionPerSection\"\x89\x01\n\x0fMetaInformation\x12\x1c\n\x14\x63reateTimeUTCSeconds\x18\x01 \x01(\r\x12\x41\n\x15supplierAndClientInfo\x18\x02 \x01(\x0b\x32\".trafficFlow.SupplierAndClientInfo\x12\x15\n\rformatVersion\x18\x03 \x01(\x05\"=\n\x15SupplierAndClientInfo\x12\x10\n\x08\x63lientID\x18\x01 \x01(\t\x12\x12\n\nsupplierID\x18\x02 \x01(\t\"u\n\x0bTrafficFlow\x12\'\n\x08location\x18\x01 \x01(\x0b\x32\x15.trafficFlow.Location\x12!\n\x05speed\x18\x02 \x01(\x0b\x32\x12.trafficFlow.Speed\x12\x1a\n\x0broadClosure\x18\x03 \x01(\x08:\x05\x66\x61lse\"\xae\x01\n\x19TrafficFlowWithPrediction\x12\'\n\x08location\x18\x01 \x01(\x0b\x32\x15.trafficFlow.Location\x12.\n\x05speed\x18\x02 \x03(\x0b\x32\x1f.trafficFlow.SpeedWithTimeStamp\x12\x1a\n\x0broadClosure\x18\x03 \x01(\x08:\x05\x66\x61lse\x12\x1c\n\x14timeToUsualInMinutes\x18\x04 \x01(\r\"\x9b\x01\n#TrafficFlowWithPredictionPerSection\x12\'\n\x08location\x18\x01 \x01(\x0b\x32\x15.trafficFlow.Location\x12-\n\x0bspeedVector\x18\x02 \x03(\x0b\x32\x18.trafficFlow.SpeedVector\x12\x1c\n\x14timeToUsualInMinutes\x18\x03 \x01(\r\"?\n\x08Location\x12\x0b\n\x03tmc\x18\x01 \x01(\t\x12\x0e\n\x06openlr\x18\x02 \x01(\x0c\x12\x16\n\x0elengthInMeters\x18\x03 \x01(\r\"\xb9\x01\n\x05Speed\x12\x18\n\x10\x61verageSpeedKmph\x18\x01 \x01(\x05\x12\x19\n\x11travelTimeSeconds\x18\x02 \x01(\x05\x12\x12\n\nconfidence\x18\x03 \x01(\x05\x12\x15\n\rrelativeSpeed\x18\x04 \x01(\x02\x12\x37\n\x10trafficCondition\x18\x05 \x01(\x0e\x32\x1d.trafficFlow.TrafficCondition\x12\x17\n\x0f\x61verageSpeedMph\x18\x06 \x01(\x05\"P\n\x12SpeedWithTimeStamp\x12\x17\n\x0fminutesInFuture\x18\x01 \x01(\r\x12!\n\x05speed\x18\x02 \x01(\x0b\x32\x12.trafficFlow.Speed\"W\n\x0bSpeedVector\x12\x17\n\x0fminutesInFuture\x18\x01 \x01(\r\x12/\n\x0csectionSpeed\x18\x02 \x03(\x0b\x32\x19.trafficFlow.SectionSpeed\"j\n\x0cSectionSpeed\x12\x1b\n\x13startOffsetInMeters\x18\x01 \x01(\r\x12!\n\x05speed\x18\x02 \x01(\x0b\x32\x12.trafficFlow.Speed\x12\x1a\n\x0broadClosure\x18\x03 \x01(\x08:\x05\x66\x61lse*\xa8\x01\n\x10TrafficCondition\x12\x17\n\x13LONG_QUEUES_TRAFFIC\x10\x01\x12\x10\n\x0c\x46REE_TRAFFIC\x10\x02\x12\x11\n\rHEAVY_TRAFFIC\x10\x03\x12\x10\n\x0cSLOW_TRAFFIC\x10\x04\x12\x13\n\x0fQUEUING_TRAFFIC\x10\x05\x12\x16\n\x12STATIONARY_TRAFFIC\x10\x06\x12\n\n\x06\x43LOSED\x10\x07\x12\x0b\n\x07UNKNOWN\x10\x08')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_TRAFFICCONDITION = _descriptor.EnumDescriptor(
  name='TrafficCondition',
  full_name='trafficFlow.TrafficCondition',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LONG_QUEUES_TRAFFIC', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FREE_TRAFFIC', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HEAVY_TRAFFIC', index=2, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SLOW_TRAFFIC', index=3, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QUEUING_TRAFFIC', index=4, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STATIONARY_TRAFFIC', index=5, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CLOSED', index=6, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=7, number=8,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1528,
  serialized_end=1696,
)
_sym_db.RegisterEnumDescriptor(_TRAFFICCONDITION)

TrafficCondition = enum_type_wrapper.EnumTypeWrapper(_TRAFFICCONDITION)
LONG_QUEUES_TRAFFIC = 1
FREE_TRAFFIC = 2
HEAVY_TRAFFIC = 3
SLOW_TRAFFIC = 4
QUEUING_TRAFFIC = 5
STATIONARY_TRAFFIC = 6
CLOSED = 7
UNKNOWN = 8



_TRAFFICFLOWGROUP = _descriptor.Descriptor(
  name='TrafficFlowGroup',
  full_name='trafficFlow.TrafficFlowGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metaInformation', full_name='trafficFlow.TrafficFlowGroup.metaInformation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='trafficFlow', full_name='trafficFlow.TrafficFlowGroup.trafficFlow', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='trafficFlowWithPrediction', full_name='trafficFlow.TrafficFlowGroup.trafficFlowWithPrediction', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='trafficFlowWithPredictionPerSection', full_name='trafficFlow.TrafficFlowGroup.trafficFlowWithPredictionPerSection', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=336,
)


_METAINFORMATION = _descriptor.Descriptor(
  name='MetaInformation',
  full_name='trafficFlow.MetaInformation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='createTimeUTCSeconds', full_name='trafficFlow.MetaInformation.createTimeUTCSeconds', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='supplierAndClientInfo', full_name='trafficFlow.MetaInformation.supplierAndClientInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='formatVersion', full_name='trafficFlow.MetaInformation.formatVersion', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=339,
  serialized_end=476,
)


_SUPPLIERANDCLIENTINFO = _descriptor.Descriptor(
  name='SupplierAndClientInfo',
  full_name='trafficFlow.SupplierAndClientInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='clientID', full_name='trafficFlow.SupplierAndClientInfo.clientID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='supplierID', full_name='trafficFlow.SupplierAndClientInfo.supplierID', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=478,
  serialized_end=539,
)


_TRAFFICFLOW = _descriptor.Descriptor(
  name='TrafficFlow',
  full_name='trafficFlow.TrafficFlow',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='location', full_name='trafficFlow.TrafficFlow.location', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='speed', full_name='trafficFlow.TrafficFlow.speed', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='roadClosure', full_name='trafficFlow.TrafficFlow.roadClosure', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=541,
  serialized_end=658,
)


_TRAFFICFLOWWITHPREDICTION = _descriptor.Descriptor(
  name='TrafficFlowWithPrediction',
  full_name='trafficFlow.TrafficFlowWithPrediction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='location', full_name='trafficFlow.TrafficFlowWithPrediction.location', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='speed', full_name='trafficFlow.TrafficFlowWithPrediction.speed', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='roadClosure', full_name='trafficFlow.TrafficFlowWithPrediction.roadClosure', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timeToUsualInMinutes', full_name='trafficFlow.TrafficFlowWithPrediction.timeToUsualInMinutes', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=661,
  serialized_end=835,
)


_TRAFFICFLOWWITHPREDICTIONPERSECTION = _descriptor.Descriptor(
  name='TrafficFlowWithPredictionPerSection',
  full_name='trafficFlow.TrafficFlowWithPredictionPerSection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='location', full_name='trafficFlow.TrafficFlowWithPredictionPerSection.location', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='speedVector', full_name='trafficFlow.TrafficFlowWithPredictionPerSection.speedVector', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timeToUsualInMinutes', full_name='trafficFlow.TrafficFlowWithPredictionPerSection.timeToUsualInMinutes', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=838,
  serialized_end=993,
)


_LOCATION = _descriptor.Descriptor(
  name='Location',
  full_name='trafficFlow.Location',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tmc', full_name='trafficFlow.Location.tmc', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='openlr', full_name='trafficFlow.Location.openlr', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='lengthInMeters', full_name='trafficFlow.Location.lengthInMeters', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=995,
  serialized_end=1058,
)


_SPEED = _descriptor.Descriptor(
  name='Speed',
  full_name='trafficFlow.Speed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='averageSpeedKmph', full_name='trafficFlow.Speed.averageSpeedKmph', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='travelTimeSeconds', full_name='trafficFlow.Speed.travelTimeSeconds', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='confidence', full_name='trafficFlow.Speed.confidence', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='relativeSpeed', full_name='trafficFlow.Speed.relativeSpeed', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='trafficCondition', full_name='trafficFlow.Speed.trafficCondition', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='averageSpeedMph', full_name='trafficFlow.Speed.averageSpeedMph', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1061,
  serialized_end=1246,
)


_SPEEDWITHTIMESTAMP = _descriptor.Descriptor(
  name='SpeedWithTimeStamp',
  full_name='trafficFlow.SpeedWithTimeStamp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='minutesInFuture', full_name='trafficFlow.SpeedWithTimeStamp.minutesInFuture', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='speed', full_name='trafficFlow.SpeedWithTimeStamp.speed', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1248,
  serialized_end=1328,
)


_SPEEDVECTOR = _descriptor.Descriptor(
  name='SpeedVector',
  full_name='trafficFlow.SpeedVector',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='minutesInFuture', full_name='trafficFlow.SpeedVector.minutesInFuture', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sectionSpeed', full_name='trafficFlow.SpeedVector.sectionSpeed', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1330,
  serialized_end=1417,
)


_SECTIONSPEED = _descriptor.Descriptor(
  name='SectionSpeed',
  full_name='trafficFlow.SectionSpeed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='startOffsetInMeters', full_name='trafficFlow.SectionSpeed.startOffsetInMeters', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='speed', full_name='trafficFlow.SectionSpeed.speed', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='roadClosure', full_name='trafficFlow.SectionSpeed.roadClosure', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1419,
  serialized_end=1525,
)

_TRAFFICFLOWGROUP.fields_by_name['metaInformation'].message_type = _METAINFORMATION
_TRAFFICFLOWGROUP.fields_by_name['trafficFlow'].message_type = _TRAFFICFLOW
_TRAFFICFLOWGROUP.fields_by_name['trafficFlowWithPrediction'].message_type = _TRAFFICFLOWWITHPREDICTION
_TRAFFICFLOWGROUP.fields_by_name['trafficFlowWithPredictionPerSection'].message_type = _TRAFFICFLOWWITHPREDICTIONPERSECTION
_METAINFORMATION.fields_by_name['supplierAndClientInfo'].message_type = _SUPPLIERANDCLIENTINFO
_TRAFFICFLOW.fields_by_name['location'].message_type = _LOCATION
_TRAFFICFLOW.fields_by_name['speed'].message_type = _SPEED
_TRAFFICFLOWWITHPREDICTION.fields_by_name['location'].message_type = _LOCATION
_TRAFFICFLOWWITHPREDICTION.fields_by_name['speed'].message_type = _SPEEDWITHTIMESTAMP
_TRAFFICFLOWWITHPREDICTIONPERSECTION.fields_by_name['location'].message_type = _LOCATION
_TRAFFICFLOWWITHPREDICTIONPERSECTION.fields_by_name['speedVector'].message_type = _SPEEDVECTOR
_SPEED.fields_by_name['trafficCondition'].enum_type = _TRAFFICCONDITION
_SPEEDWITHTIMESTAMP.fields_by_name['speed'].message_type = _SPEED
_SPEEDVECTOR.fields_by_name['sectionSpeed'].message_type = _SECTIONSPEED
_SECTIONSPEED.fields_by_name['speed'].message_type = _SPEED
DESCRIPTOR.message_types_by_name['TrafficFlowGroup'] = _TRAFFICFLOWGROUP
DESCRIPTOR.message_types_by_name['MetaInformation'] = _METAINFORMATION
DESCRIPTOR.message_types_by_name['SupplierAndClientInfo'] = _SUPPLIERANDCLIENTINFO
DESCRIPTOR.message_types_by_name['TrafficFlow'] = _TRAFFICFLOW
DESCRIPTOR.message_types_by_name['TrafficFlowWithPrediction'] = _TRAFFICFLOWWITHPREDICTION
DESCRIPTOR.message_types_by_name['TrafficFlowWithPredictionPerSection'] = _TRAFFICFLOWWITHPREDICTIONPERSECTION
DESCRIPTOR.message_types_by_name['Location'] = _LOCATION
DESCRIPTOR.message_types_by_name['Speed'] = _SPEED
DESCRIPTOR.message_types_by_name['SpeedWithTimeStamp'] = _SPEEDWITHTIMESTAMP
DESCRIPTOR.message_types_by_name['SpeedVector'] = _SPEEDVECTOR
DESCRIPTOR.message_types_by_name['SectionSpeed'] = _SECTIONSPEED
DESCRIPTOR.enum_types_by_name['TrafficCondition'] = _TRAFFICCONDITION

TrafficFlowGroup = _reflection.GeneratedProtocolMessageType('TrafficFlowGroup', (_message.Message,), dict(
  DESCRIPTOR = _TRAFFICFLOWGROUP,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.TrafficFlowGroup)
  ))
_sym_db.RegisterMessage(TrafficFlowGroup)

MetaInformation = _reflection.GeneratedProtocolMessageType('MetaInformation', (_message.Message,), dict(
  DESCRIPTOR = _METAINFORMATION,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.MetaInformation)
  ))
_sym_db.RegisterMessage(MetaInformation)

SupplierAndClientInfo = _reflection.GeneratedProtocolMessageType('SupplierAndClientInfo', (_message.Message,), dict(
  DESCRIPTOR = _SUPPLIERANDCLIENTINFO,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.SupplierAndClientInfo)
  ))
_sym_db.RegisterMessage(SupplierAndClientInfo)

TrafficFlow = _reflection.GeneratedProtocolMessageType('TrafficFlow', (_message.Message,), dict(
  DESCRIPTOR = _TRAFFICFLOW,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.TrafficFlow)
  ))
_sym_db.RegisterMessage(TrafficFlow)

TrafficFlowWithPrediction = _reflection.GeneratedProtocolMessageType('TrafficFlowWithPrediction', (_message.Message,), dict(
  DESCRIPTOR = _TRAFFICFLOWWITHPREDICTION,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.TrafficFlowWithPrediction)
  ))
_sym_db.RegisterMessage(TrafficFlowWithPrediction)

TrafficFlowWithPredictionPerSection = _reflection.GeneratedProtocolMessageType('TrafficFlowWithPredictionPerSection', (_message.Message,), dict(
  DESCRIPTOR = _TRAFFICFLOWWITHPREDICTIONPERSECTION,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.TrafficFlowWithPredictionPerSection)
  ))
_sym_db.RegisterMessage(TrafficFlowWithPredictionPerSection)

Location = _reflection.GeneratedProtocolMessageType('Location', (_message.Message,), dict(
  DESCRIPTOR = _LOCATION,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.Location)
  ))
_sym_db.RegisterMessage(Location)

Speed = _reflection.GeneratedProtocolMessageType('Speed', (_message.Message,), dict(
  DESCRIPTOR = _SPEED,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.Speed)
  ))
_sym_db.RegisterMessage(Speed)

SpeedWithTimeStamp = _reflection.GeneratedProtocolMessageType('SpeedWithTimeStamp', (_message.Message,), dict(
  DESCRIPTOR = _SPEEDWITHTIMESTAMP,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.SpeedWithTimeStamp)
  ))
_sym_db.RegisterMessage(SpeedWithTimeStamp)

SpeedVector = _reflection.GeneratedProtocolMessageType('SpeedVector', (_message.Message,), dict(
  DESCRIPTOR = _SPEEDVECTOR,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.SpeedVector)
  ))
_sym_db.RegisterMessage(SpeedVector)

SectionSpeed = _reflection.GeneratedProtocolMessageType('SectionSpeed', (_message.Message,), dict(
  DESCRIPTOR = _SECTIONSPEED,
  __module__ = 'ProtobufTrafficFlow_v5_pb2'
  # @@protoc_insertion_point(class_scope:trafficFlow.SectionSpeed)
  ))
_sym_db.RegisterMessage(SectionSpeed)


# @@protoc_insertion_point(module_scope)
